/* FILE: DynWaitlist.h
 *
 * WHAT: Declares a dynamic list of wait handles to be used with WaitForSingleObjects.
 *       Each handle is assigned an ID, and signaling can be checked against the ID
 *       rather than the index, making the code more maintainable and 
 *       flexible - especially in the case where the list of handles to wait for
 *       changes over time.
 */


#ifndef __DYNWAITLIST_H__
#define __DYNWAITLIST_H__

#include "windows.h"

// bits to be combined with handle IDs to detect the state
#define HANDLE_SIGNALED   0x10000000
#define HANDLE_ABANDONED  0x20000000


// translated result to be compared against:
//	HANDLE_SIGNALED|id    (for each id)
//  HANDLE_ABANDONED|id   (for each id)
//  MsgArrival()          // shortcut WAIT_OBJECT_0 + nhandles
//  WAIT_TIMEOUT
//  WAIT_FAILED
//  WAIT_IO_COMPLETION

class DynWaitlistImpl
{
protected: 
	DynWaitlistImpl( WORD nMaxHandles, HANDLE *pHandles, WORD *pIds, WORD wFirstId );
	virtual ~DynWaitlistImpl() { }

public:
	// adds a handle to the list. Returns TRUE if successful
	BOOL Add( HANDLE hNewHandle, WORD *pwNewId );

	// removes a handle from the list. Returns TRUE if successful
	BOOL Remove( WORD wId );

	DWORD Wait(DWORD dwTimeoutMs, BOOL bWaitAll = FALSE)
	{
		return TranslateWaitResult(
			WaitForMultipleObjects( m_nHandles,
									m_pHandles,
									bWaitAll,
									dwTimeoutMs )
	                             );
	}

#ifndef _WIN32_WCE
	DWORD WaitEx( DWORD dwTimeoutMs, BOOL bAlertable, BOOL bWaitAll = FALSE )
	{
		return TranslateWaitResult(
			WaitForMultipleObjectsEx(
			  m_nHandles,
			  m_pHandles,
			  bWaitAll,
			  dwTimeoutMs,
			  bAlertable )
			  );
	}
#endif

	DWORD MsgWait(DWORD dwTimeoutMs, DWORD dwWakemask, BOOL bWaitAll = FALSE)
	{
		return TranslateWaitResult(
			MsgWaitForMultipleObjects(
			  m_nHandles,
			  m_pHandles,
			  bWaitAll,
			  dwTimeoutMs,
			  dwWakemask )
            );
	}
	DWORD MsgWaitEx(DWORD dwTimeoutMs, DWORD dwWakemask, DWORD dwFlags = 0)
	{
		return TranslateWaitResult(
			MsgWaitForMultipleObjectsEx(
			  m_nHandles,
			  m_pHandles,
			  dwTimeoutMs,
			  dwWakemask,
			  dwFlags )
			);
	}

	DWORD TranslateWaitResult( DWORD dwWaitResult );

	DWORD MsgArrival() const { return WAIT_OBJECT_0 + m_nHandles; }

	// useful only if user wants to get access to the list
	HANDLE *getHandles() { return m_pHandles; }
	DWORD getNHandles() { return m_nHandles; }
    BOOL isFull() { return m_nHandles >= m_nMaxHandles; }

private:
	HANDLE *m_pHandles;
	WORD *m_pIds;
	WORD m_nHandles;
	WORD m_nMaxHandles;
};


template <int _nMaxHandles> class DynWaitlist: public DynWaitlistImpl
{
public:
	DynWaitlist(WORD wFirstId): DynWaitlistImpl( _nMaxHandles, handles, ids, wFirstId ) { }
	virtual ~DynWaitlist() { }

private:
	HANDLE handles[ _nMaxHandles ];
	WORD ids[ _nMaxHandles ];
};




#endif
